@@ -139,8 +139,7 @@ ienkf.internal <- function (object, Nenkf, rw.sd,
   if (is.null(Np)) {
     pStop_(sQuote("Np")," must be specified.")
   }  else if (!is.numeric(Np)) {
-    pStop_(sQuote("Np"),
-      " must be a number, a vector of numbers, or a function.")
+    pStop_(sQuote("Np")," must be a number or a vector of numbers")
   }
 
   Np <- as.integer(Np)
@@ -233,8 +232,7 @@ ienkf.filter <- function (object, params, Np, enkfiter, rw.sd, cooling.fn,
   Np <- as.integer(Np)
 
   do_ta <- length(.indices)>0L
-  if (do_ta && length(.indices)!=Np)
-    pStop_(sQuote(".indices")," has improper length.")
+  if (do_ta) pStop_(sQuote(".indices")," for ancestor tracking is not supported.")
 
   times <- time(object,t0=TRUE)
   t <- time(object)
@@ -295,19 +293,17 @@ ienkf.filter <- function (object, params, Np, enkfiter, rw.sd, cooling.fn,
     R <- diag(rowMeans(meas_var))
     sqrtR <- tryCatch(
       t(chol(R)),                     # t(sqrtR)%*%sqrtR == R
-      error = function (e) {
-        pStop_("degenerate ",sQuote("R"), "at time ", sQuote(nt), ": ",conditionMessage(e))
-      }
+      error = function (e) pStop_("degenerate ",sQuote("R"), "at time ", sQuote(nt), ": ",conditionMessage(e))
     )
 
-                                        # expand the state space
+    # expand the state space
     XT <- rbind(X[,,1],params)
     pm <- rowMeans(XT) # prediction mean
 
-                                        # forecast mean
+    # forecast mean
     ym <- rowMeans(Y)
 
-                                        # center prediction and forecast ensembles
+    # center prediction and forecast ensembles
     XT <- XT-pm
     Y <- Y-ym
 
@@ -322,8 +318,8 @@ ienkf.filter <- function (object, params, Np, enkfiter, rw.sd, cooling.fn,
     XT <- XT+pm+crossprod(Kt,resid-Y+Ek)
     params <- XT[pnames,,drop = FALSE]
     X <- XT[xnames,,drop = FALSE]
-    loglik[nt] <- sum(dnorm(x=crossprod(svdS$u,resid),mean=0,sd=sqrt(svdS$d),log=TRUE))
-                                        # print(rowMeans(partrans(object,params,dir="fromEst",.gnsi=gnsi)))
+    loglik[nt] <- sum(dnorm(x=crossprod(svdS$u,resid),
+      mean=0,sd=sqrt(svdS$d),log=TRUE))
 
     ## compute mean at last timestep
     if (nt == ntimes) {
