@@ -72,7 +72,7 @@ setMethod(
   "igirf",
   signature=signature(data="missing"),
   definition=function (...) {
-    stop("igirf: ","data"," is a required argument.")
+    pStop_("igirf: ","data"," is a required argument.")
   }
 )
 
@@ -84,7 +84,7 @@ setMethod(
   "igirf",
   signature=signature(data="ANY"),
   definition=function (data, ...) {
-    stop("igirf is undefined for ", sQuote(data), "of class ", sQuote(class(data)), ".")
+    pStop_("igirf is undefined for ", sQuote("data"), "of class ", sQuote(class(data)), ".")
   }
 )
 
@@ -97,15 +97,25 @@ setMethod(
   signature=signature(data="spatPomp"),
   definition=function (data,Ngirf,Np,rw.sd,cooling.type,cooling.fraction.50,
     Ninter,lookahead=1,Nguide,kind=c('bootstrap', 'moment'),
-    tol = 1e-300,
+    tol = 1e-100,
     ..., verbose = getOption("verbose", FALSE)) {
+
+    ep <- paste0("in ", sQuote("igirf") , " : ")
     if(missing(Ninter)) Ninter <- length(unit_names(data))
+    if(missing(Ngirf)) pStop_(ep, sQuote("Ngirf"), " is a required argument")
+    if (missing(rw.sd)) pStop_(ep, sQuote("rw.sd")," must be specified.")
+    if (missing(cooling.fraction.50))
+      pStop_(ep, sQuote("cooling.fraction.50")," is a required argument.")
+    if (missing(cooling.type))
+      pStop_(ep, sQuote("cooling.type")," is a required argument.")
+    if (missing(Np)) pStop_(ep, sQuote("Np")," must be specified.")
+    if (missing(Nguide))  pStop_(ep, sQuote("Nguide")," must be specified.")
     kind = match.arg(kind)
     tryCatch(
       igirf.internal(data,Ngirf,Np,rw.sd,cooling.type,cooling.fraction.50,
         Ninter,lookahead,Nguide,kind,tol = tol,
         ...,verbose=verbose),
-      error = function (e) stop("igirf: ", e)
+      error = function (e) pStop_("in ", sQuote("igirf") , " : ", e)
     )
   }
 )
@@ -155,13 +165,14 @@ igirf.internal <- function (object,Ngirf,Np,rw.sd,cooling.type,cooling.fraction.
     unitname=object@unitname,
     unit_statenames=object@unit_statenames,
     unit_obsnames = object@unit_obsnames)
-  if (undefined(object@rprocess) || undefined(object@dmeasure))
-    stop(paste(sQuote(c("rprocess","dmeasure")),collapse=", ")," are needed basic components.")
+
+if (undefined(object@rprocess) || undefined(object@dmeasure))
+    pStop_(sQuote(c("rprocess","dmeasure"))," are needed basic components.")
 
   gnsi <- as.logical(.gnsi)
 
   if (length(Ngirf) != 1 || !is.numeric(Ngirf) || !is.finite(Ngirf) || Ngirf < 1)
-    stop(sQuote("Ngirf")," must be a positive integer.")
+    pStop_(sQuote("Ngirf")," must be a positive integer.")
   Ngirf <- as.integer(Ngirf)
 
   if (is.null(.paramMatrix)) {
@@ -170,53 +181,24 @@ igirf.internal <- function (object,Ngirf,Np,rw.sd,cooling.type,cooling.fraction.
     start <- apply(.paramMatrix,1L,mean)
   }
 
-  ntimes <- length(time(object))
-
-  if (is.null(Np)) {
-    stop(sQuote("Np")," must be specified.")
-  } else if (is.function(Np)) {
-    Np <- tryCatch(
-      vapply(seq_len(ntimes),Np,numeric(1)),
-      error = function (e) {
-        stop("if ",sQuote("Np"),
-          " is a function, it must return a single positive integer.")
-      }
-    )
-  } else if (!is.numeric(Np)) {
-    stop(sQuote("Np"),
-      " must be a number, a vector of numbers, or a function.")
+  if (missing(Np) || is.null(Np)) {
+    pStop_(sQuote("Np")," must be specified.")
+  } else if (length(Np)>1) {
+    pStop_(sQuote("Np")," must be a single positive integer")
+  } else if (!is.numeric(Np)|| !is.finite(Np) || (Np < 0)) {
+    pStop_(sQuote("Np")," must be a single positive integer")
   }
 
-  if (length(Np) == 1) {
-    Np <- rep(Np,times=ntimes)
-  } else if (length(Np) > ntimes) {
-    if (Np[1L] != Np[ntimes+1] || length(Np) > ntimes+1) {
-      warning("in igirf ","Np[k] ignored for k > ",sQuote("length(time(object))"),".")
-    }
-    Np <- head(Np,ntimes)
-  } else if (length(Np) < ntimes) {
-    stop(sQuote("Np")," must have length 1 or ",
-      sQuote("length(time(object))"),".")
-  }
-
-  if (!all(is.finite(Np)) || any(Np <= 0))
-    stop(sQuote("Np")," must be a positive integer.")
-
   Np <- as.integer(Np)
-  Np <- c(Np,Np[1L])
 
-  if (missing(rw.sd))
-    stop(sQuote("rw.sd")," must be specified!")
   ##rw.sd <- perturbn.kernel.sd(rw.sd,time=time(object),paramnames=names(start))
   rw.sd <- perturbn.kernel.sd(rw.sd,
     time=igirf_rw_sd_times(times=time(object),Ninter=Ninter),
     paramnames=names(start))
-  if (missing(cooling.fraction.50))
-    stop(sQuote("cooling.fraction.50")," is a required argument.")
   if (length(cooling.fraction.50) != 1 || !is.numeric(cooling.fraction.50) ||
         !is.finite(cooling.fraction.50) || cooling.fraction.50 <= 0 ||
           cooling.fraction.50 > 1)
-    stop(sQuote("cooling.fraction.50")," must be in (0,1].")
+    pStop_(sQuote("cooling.fraction.50")," must be in (0,1].")
   cooling.fraction.50 <- as.numeric(cooling.fraction.50)
 
   cooling.fn <- mif2.cooling(
@@ -226,7 +208,7 @@ igirf.internal <- function (object,Ngirf,Np,rw.sd,cooling.type,cooling.fraction.
   )
 
   if (is.null(.paramMatrix)) {
-    paramMatrix <- array(data=start,dim=c(length(start),Np[1L]),
+    paramMatrix <- array(data=start,dim=c(length(start),Np),
       dimnames=list(variable=names(start),rep=NULL))
   } else {
     paramMatrix <- .paramMatrix
@@ -294,26 +276,26 @@ igirf.momgirf <- function (object, params, Ninter, lookahead, Nguide,
   gnsi <- as.logical(.gnsi)
   verbose <- as.logical(verbose)
   girfiter <- as.integer(girfiter)
-  Np <- as.integer(Np)
-  ep <- paste0("in ",sQuote("igirf"),": ")
+  ep <- paste0("in ",sQuote("igirf"), " (method=moment) : ")
 
   if (length(tol) != 1 || !is.finite(tol) || tol < 0)
-    stop(sQuote("tol")," should be a small positive number.")
+    pStop_(ep, sQuote("tol")," should be a small positive number.")
 
-  do_ta <- length(.indices)>0L
-  if (do_ta && length(.indices)!=Np[1L])
-    stop(sQuote(".indices")," has improper length.")
+  ## ancestor tracking is not supported
+  ## do_ta <- length(.indices)>0L
+  ## if (do_ta && length(.indices)!=Np)
+  ##   pStop_(ep, sQuote(".indices")," has improper length.")
 
   times <- time(object,t0=TRUE)
   ntimes <- length(times)-1
-  nunits <- length(unit_names(object))
+  ## nunits <- length(unit_names(object))
 
   cond.loglik <- array(0, dim = c(ntimes, Ninter))
 
   znames <- object@accumvars
 
-                                        # Initialize filter guide function
-  log_filter_guide_fun <- array(0, dim = Np[1])
+  # Initialize filter guide function
+  log_filter_guide_fun <- array(0, dim = Np)
   for (nt in 0:(ntimes-1)) {
     pmag <- cooling.fn(nt,girfiter)$alpha*rw.sd[,nt]
     params <- .Call(randwalk_perturbation_spatPomp,params,pmag)
@@ -327,7 +309,7 @@ igirf.momgirf <- function (object, params, Ninter, lookahead, Nguide,
     lookahead_steps <- min(lookahead, ntimes-nt)
     ## Four-dimensional array: nvars by nguide by ntimes by nreps
     Xg <- array(0,
-      dim=c(length(statenames), Nguide, lookahead_steps, Np[1]),
+      dim=c(length(statenames), Nguide, lookahead_steps, Np),
       dimnames = list(
         nvars = statenames,
         ng = NULL,
@@ -335,30 +317,30 @@ igirf.momgirf <- function (object, params, Ninter, lookahead, Nguide,
         nreps = NULL
       )
     )
-                                        # For each particle get K guide particles, and fill in sample variance over K for each (lookahead value - unit - particle) combination
-    fcst_samp_var <- array(0, dim = c(length(unit_names(object)), lookahead_steps, Np[1]))
-    x_with_guides <- x[,rep(1:Np[1], rep(Nguide, Np[1]))]
-    tp_with_guides <- tparams[,rep(1:Np[1], rep(Nguide, Np[1]))]
-    Xg <- rprocess(object, x0=x_with_guides, t0=times[nt+1], times=times[(nt+2):(nt+1+lookahead_steps)],
+    # For each particle get K guide particles, and fill in sample variance
+    # over K for each (lookahead value - unit - particle) combination
+    fcst_samp_var <- array(0, dim = c(length(unit_names(object)), lookahead_steps, Np))
+    x_with_guides <- x[,rep(1:Np, rep(Nguide, Np))]
+    tp_with_guides <- tparams[,rep(1:Np, rep(Nguide, Np))]
+    Xg <- rprocess(object, x0=x_with_guides, t0=times[nt+1],
+      times=times[(nt+2):(nt+1+lookahead_steps)],
       params=tp_with_guides,.gnsi=gnsi)
     xx <- tryCatch(
       .Call(do_fcst_samp_var,
         object=object,
         X=Xg,
-        Np = as.integer(Np[1]),
+        Np = as.integer(Np),
         times=times[(nt+2):(nt+1+lookahead_steps)],
         params=tp_with_guides,
         gnsi=TRUE),
-      error = function (e) {
-        stop(ep,conditionMessage(e),call.=FALSE) # nocov
-      }
+      error = function (e) pStop_("in ", sQuote("igirf_moment"), " : ", conditionMessage(e)) 
     )
     fcst_samp_var <- xx
-    dim(fcst_samp_var) <- c(length(unit_names(object)), lookahead_steps, Np[1])
+    dim(fcst_samp_var) <- c(length(unit_names(object)), lookahead_steps, Np)
 
     for (s in 1:Ninter){
       tparams <- partrans(object,params,dir="fromEst",.gnsi=gnsi)
-                                        # Get prediction simulations: nvars by nreps by 1 array
+      # Get prediction simulations: nvars by nreps by 1 array
       X <- rprocess(object,x0=x, t0 = tt[s], times= tt[s+1],
         params=tparams,.gnsi=gnsi)
       if(s>1 && length(znames)>0){
@@ -385,37 +367,33 @@ igirf.momgirf <- function (object, params, Ninter, lookahead, Nguide,
         .Call(do_vunit_measure,
           object=object,
           X=skel,
-          Np = as.integer(Np[1]),
+          Np = as.integer(Np),
           times=times[(nt+2):(nt+1+lookahead_steps)],
           params=tparams,
           gnsi=TRUE),
-        error = function (e) {
-          stop(ep,conditionMessage(e),call.=FALSE) # nocov
-        }
+        error = function (e) pStop_("in ", sQuote("igirf_moment"), " : ",conditionMessage(e)) 
       )
-      dim(meas_var_skel) <- c(length(unit_names(object)), lookahead_steps, Np[1])
-      fcst_var_upd <- array(0, dim = c(length(unit_names(object)), lookahead_steps, Np[1]))
+      dim(meas_var_skel) <- c(length(unit_names(object)), lookahead_steps, Np)
+      fcst_var_upd <- array(0, dim = c(length(unit_names(object)), lookahead_steps, Np))
       for(l in 1:lookahead_steps) fcst_var_upd[,l,] <- fcst_samp_var[,l,]*(times[nt+1+l] - tt[s+1])/(times[nt+1+l] - times[nt+1])
       inflated_var <- meas_var_skel + fcst_var_upd
-      array.tparams <- array(NA, dim = c(dim(tparams)[1], length(unit_names(object)), Np[1], lookahead_steps), dimnames = list(tparams = rownames(tparams)))
+      array.tparams <- array(NA, dim = c(dim(tparams)[1], length(unit_names(object)), Np, lookahead_steps), dimnames = list(tparams = rownames(tparams)))
       for(i in 1:length(unit_names(object))) array.tparams[,i,,1:lookahead_steps] <- tparams
       mmp <- tryCatch(
         .Call(do_munit_measure,
           object=object,
           X=skel,
           vc=inflated_var,
-          Np = as.integer(Np[1]),
+          Np = as.integer(Np),
           times=times[(nt+2):(nt+1+lookahead_steps)],
           params=array.tparams,
           gnsi=TRUE),
-        error = function (e) {
-          stop(ep,conditionMessage(e),call.=FALSE) # nocov
-        }
+        error = function (e) pStop_("in ", sQuote("igirf_moment"), " : ", conditionMessage(e))
       )
       mom_match_param <- mmp
-      dim(mom_match_param) <- c(dim(tparams)[1], length(unit_names(object)), lookahead_steps, Np[1])
+      dim(mom_match_param) <- c(dim(tparams)[1], length(unit_names(object)), lookahead_steps, Np)
       dimnames(mom_match_param) <- list(tparam = rownames(tparams))
-      log_guide_fun <- vector(mode = "numeric", length = Np[1]) + 1
+      log_guide_fun <- vector(mode = "numeric", length = Np) + 1
 
       for(l in 1:lookahead_steps){
         if(nt+1+l-lookahead <= 0) discount_denom_init <- object@t0
@@ -431,23 +409,22 @@ igirf.momgirf <- function (object, params, Ninter, lookahead, Nguide,
           log=TRUE,
           .gnsi=gnsi
         )),
-        error = function (e) {
-          stop(ep,"error in calculation of log_dmeas_weights: ",
-            conditionMessage(e),call.=FALSE)
-        }
+        error = function (e) pStop_("in igirf_moment : error in calculation of log_dmeas_weights: ", conditionMessage(e))
         )
-        if(any(is.na(log_dmeas_weights))){
-                                        # find particle with the NA
-          na_ix <- which(is.na(log_dmeas_weights[,,1]))[1]
-          na_ix_col <- (na_ix %/% nunits) + (na_ix %% nunits > 0)
-          illegal_dunit_measure_error(
-            time=times[nt+1+l],
-            lik=log_dmeas_weights[,na_ix_col,1,drop=FALSE],
-            datvals=object@data[,nt+l],
-            states=skel[,na_ix_col,l],
-            params=mom_match_param[,,l,na_ix_col]
-          )
-        }
+
+        ## uncomment for debugging: tracking down particles with NA weight
+        ## if(any(is.na(log_dmeas_weights))){
+        ##   na_ix <- which(is.na(log_dmeas_weights[,,1]))[1]
+        ##   na_ix_col <- (na_ix %/% nunits) + (na_ix %% nunits > 0)
+        ##   illegal_dunit_measure_error(
+        ##     time=times[nt+1+l],
+        ##     lik=log_dmeas_weights[,na_ix_col,1,drop=FALSE],
+        ##     datvals=object@data[,nt+l],
+        ##     states=skel[,na_ix_col,l],
+        ##     params=mom_match_param[,,l,na_ix_col]
+        ##   )
+        ##  }
+	
         log_resamp_weights <- apply(log_dmeas_weights[,,1,drop=FALSE], 2, function(x) sum(x))*discount_factor
         log_guide_fun <- log_guide_fun + log_resamp_weights
       }
@@ -469,10 +446,7 @@ igirf.momgirf <- function (object, params, Ninter, lookahead, Nguide,
             log=TRUE,
             .gnsi=gnsi
           ),
-          error = function (e) {
-            stop(ep,"error in calculation of log_meas_weights: ",
-              conditionMessage(e),call.=FALSE)
-          }
+          error = function (e) pStop_("in igirf_moment : error in calculation of log_meas_weights: ", conditionMessage(e))
         )
         gnsi <- FALSE
         log_weights <- as.numeric(log_meas_weights) + log_s_not_1_weights
@@ -481,7 +455,7 @@ igirf.momgirf <- function (object, params, Ninter, lookahead, Nguide,
         if (any(log_weights>-Inf)) {
           coef(object,transform=TRUE) <- apply(params,1L,weighted.mean,w=exp(log_weights))
         } else {
-          warning("igirf: ","filtering failure at last filter iteration; using ",
+          warning("igirf_moment: ","filtering failure at last filter iteration; using ",
             "unweighted mean for point estimate.")
           coef(object,transform=TRUE) <- apply(params,1L,mean)
         }
@@ -494,7 +468,7 @@ igirf.momgirf <- function (object, params, Ninter, lookahead, Nguide,
           .Call(girf_computations,
             x=X,
             params=params,
-            Np=Np[nt+1],
+            Np=Np,
             trackancestry=FALSE,
             doparRS=TRUE,
             weights=weights,
@@ -502,9 +476,7 @@ igirf.momgirf <- function (object, params, Ninter, lookahead, Nguide,
             fsv=fcst_samp_var,
             tol=tol
           ),
-          error = function (e) {
-            stop(ep,conditionMessage(e),call.=FALSE) # nocov
-          }
+          error = function (e) pStop_("in igirf_moment : ", conditionMessage(e))
         )
         cond.loglik[nt+1, s] <- xx$loglik + max_log_weights
         x <- xx$states
@@ -527,7 +499,7 @@ igirf.momgirf <- function (object, params, Ninter, lookahead, Nguide,
     Nguide=Nguide,
     lookahead=lookahead,
     paramMatrix=params,
-    Np=Np[1],
+    Np=Np,
     tol=tol,
     loglik=sum(cond.loglik)
   )
@@ -542,14 +514,15 @@ igirf.bootgirf <- function (object, params, Ninter, lookahead, Nguide,
   verbose <- as.logical(verbose)
   girfiter <- as.integer(girfiter)
   Np <- as.integer(Np)
-  ep <- paste0("in ",sQuote("ibootgirf"),": ")
+  ep <- paste0("in ",sQuote("igirf")," (method=bootstrap) : ")
 
   if (length(tol) != 1 || !is.finite(tol) || tol < 0)
     stop(ep,sQuote("tol")," should be a small positive number.",call.=FALSE)
 
-  do_ta <- length(.indices)>0L
-  if (do_ta && length(.indices)!=Np[1L])
-    stop(ep,sQuote(".indices")," has improper length.",call.=FALSE)
+  ## ancestor tracking is not supported
+  ## do_ta <- length(.indices)>0L
+  ## if (do_ta && length(.indices)!=Np)
+  ##   pStop_(ep,sQuote(".indices")," has improper length.")
 
   times <- time(object,t0=TRUE)
   ntimes <- length(times)-1
@@ -559,7 +532,7 @@ igirf.bootgirf <- function (object, params, Ninter, lookahead, Nguide,
 
   znames <- object@accumvars
 
-  log_filter_guide_fun <- array(0, dim = Np[1])
+  log_filter_guide_fun <- array(0, dim = Np)
   for (nt in 0:(ntimes-1)) {
     pmag <- cooling.fn(nt,girfiter)$alpha*rw.sd[,nt]
     params <- .Call(randwalk_perturbation_spatPomp,params,pmag)
@@ -572,7 +545,7 @@ igirf.bootgirf <- function (object, params, Ninter, lookahead, Nguide,
     lookahead_steps <- min(lookahead, ntimes-nt)
     ## Four-dimensional array: nvars by nguide by ntimes by nreps
     Xg <- array(0,
-      dim=c(length(statenames), Nguide, lookahead_steps, Np[1]),
+      dim=c(length(statenames), Nguide, lookahead_steps, Np),
       dimnames = list(
         nvars = statenames,
         ng = NULL,
@@ -580,7 +553,7 @@ igirf.bootgirf <- function (object, params, Ninter, lookahead, Nguide,
         nreps = NULL
       )
     )
-    guidesim_index <- seq_len(Np[1]) ## the index for guide simulations (to be updated each time resampling occurs)
+    guidesim_index <- seq_len(Np) ## the index for guide simulations (to be updated each time resampling occurs)
     x_with_guides <- x[,rep(guidesim_index, each = Nguide)]
     tp_with_guides <- tparams[,rep(guidesim_index, each = Nguide)]
     Xg <- rprocess(object, x0=x_with_guides, t0=times[nt+1], times=times[(nt+2):(nt+1+lookahead_steps)],
@@ -591,11 +564,11 @@ igirf.bootgirf <- function (object, params, Ninter, lookahead, Nguide,
       params=tparams,
       times = times[(nt+2):(nt+1+lookahead_steps)],
       method = 'adams')
-    resids <- Xg - Xskel[,rep(1:Np[1], each=Nguide),,drop=FALSE] # residuals
+    resids <- Xg - Xskel[,rep(1:Np, each=Nguide),,drop=FALSE] # residuals
     rm(Xg, Xskel, x_with_guides)
     for (s in 1:Ninter){
       tparams <- partrans(object,params,dir="fromEst",.gnsi=gnsi)
-      tp_with_guides <- tparams[,rep(1:Np[1], rep(Nguide, Np[1]))]
+      tp_with_guides <- tparams[,rep(1:Np, rep(Nguide, Np))]
       X <- rprocess(object,x0=x, t0 = tt[s], times= tt[s+1],
         params=tparams,.gnsi=gnsi)
       if(s>1 && length(znames)>0){
@@ -620,51 +593,63 @@ igirf.bootgirf <- function (object, params, Ninter, lookahead, Nguide,
       } else {
         skel <- X
       }
-      log_guide_fun <- vector(mode = "numeric", length = Np[1]) + 1
+      log_guide_fun <- vector(mode = "numeric", length = Np) + 1
 
-      for(l in 1:lookahead_steps){
-        if(nt+1+l-lookahead <= 0) discount_denom_init <- object@t0
-        else discount_denom_init <- times[nt+1+l - lookahead]
-        discount_factor <- 1 - (times[nt+1+l] - tt[s+1])/(times[nt+1+l] - discount_denom_init)/ifelse(lookahead==1,2,1) ## to ensure that the discount factor does not become too small for L=1 and small s (which can lead to very uninformative guide function), increase the discount factor to at least 1/2 when L=1.
+      for(lag in 1:lookahead_steps){
+        if(nt+1+lag-lookahead <= 0) discount_denom_init <- object@t0
+        else discount_denom_init <- times[nt+1+lag - lookahead]
+        discount_factor <- 1 - (times[nt+1+lag] - tt[s+1])/(times[nt+1+lag] - discount_denom_init)/ifelse(lookahead==1,2,1)
+	## to ensure that the discount factor does not become too small for L=1
+	## and small s (which can lead to very uninformative guide function), increase
+	## the discount factor to at least 1/2 when L=1.
 
-                                        # construct pseudo-simulations by adding simulated noise terms (residuals) to the skeletons
-        pseudosims <- skel[,rep(1:Np[1], each=Nguide),l,drop=FALSE] +
-          resids[,rep(guidesim_index-1, each=Nguide)*Nguide+rep(1:Nguide, Np[1]),l,drop=FALSE] -
-          resids[,rep(guidesim_index-1, each=Nguide)*Nguide+rep(1:Nguide, Np[1]),1,drop=FALSE] +
-          resids[,rep(guidesim_index-1, each=Nguide)*Nguide+rep(1:Nguide, Np[1]),1,drop=FALSE] * sqrt((times[nt+2]-tt[s+1])/(times[nt+2]-times[nt+1]))
-        rm(skel)
+        ## construct pseudo-simulations by adding simulated noise terms (residuals)
+	## to the skeletons
+        pseudosims <- skel[,rep(1:Np, each=Nguide),lag,drop=FALSE] +
+          resids[,rep(guidesim_index-1, each=Nguide)*
+	    Nguide+rep(1:Nguide, Np),lag,drop=FALSE] -
+          resids[,rep(guidesim_index-1, each=Nguide)*
+	    Nguide+rep(1:Nguide, Np),1,drop=FALSE] +
+          resids[,rep(guidesim_index-1, each=Nguide)*
+	    Nguide+rep(1:Nguide, Np),1,drop=FALSE] *
+	      sqrt((times[nt+2]-tt[s+1])/(times[nt+2]-times[nt+1]))
         log_dmeas_weights <- tryCatch(
         (vec_dmeasure(
           object,
-          y=object@data[,nt+l,drop=FALSE],
+          y=object@data[,nt+lag,drop=FALSE],
           x=pseudosims,
-          times=times[nt+1+l],
+          times=times[nt+1+lag],
           params=tp_with_guides,
           log=TRUE,
           .gnsi=gnsi
         )),
-        error = function (e) {
-          stop(ep,"error in calculation of log_dmeas_weights: ",
-            conditionMessage(e),call.=FALSE)
-        }
+        error = function (e) pStop_("error in igirf_bootstrap calculation of log_dmeas_weights: ", conditionMessage(e))
         )
-        if(any(is.na(log_dmeas_weights))){
-                                        # find particle with the NA
-          na_ix <- which(is.na(log_dmeas_weights[,,1]))[1]
-          na_ix_col <- (na_ix %/% nunits) + (na_ix %% nunits > 0)
-          illegal_dunit_measure_error(
-            time=times[nt+1+l],
-            lik=log_dmeas_weights[,na_ix_col,1,drop=FALSE],
-            datvals=object@data[,nt+l],
-            states=pseudosims[,na_ix_col,1L],
-            params=tp_with_guides[,na_ix_col]
-          )
-        }
-        ldw <- array(log_dmeas_weights, c(nunits,Nguide,Np[1])) # log_dmeas_weights is an array with dim U*(Np*Nguide)*1. Reorder it as U*Nguide*Np
-        log_fcst_lik <- colSums(log(apply(exp(ldw),c(1,3),sum)/Nguide)) # average dmeas (natural scale) over Nguide sims, then take log, and then sum over 1:U (for each particle)
+
+        ## uncomment for debugging: tracking down particles with NA weight
+        ## if(any(is.na(log_dmeas_weights))){
+        ##   na_ix <- which(is.na(log_dmeas_weights[,,1]))[1]
+        ##   na_ix_col <- (na_ix %/% nunits) + (na_ix %% nunits > 0)
+        ##   illegal_dunit_measure_error(
+        ##     time=times[nt+1+lag],
+        ##     lik=log_dmeas_weights[,na_ix_col,1,drop=FALSE],
+        ##     datvals=object@data[,nt+lag],
+        ##     states=pseudosims[,na_ix_col,1L],
+        ##     params=tp_with_guides[,na_ix_col]
+        ##   )
+        ## }
+
+        ## log_dmeas_weights is an array with dim U*(Np*Nguide)*1.
+	## Reorder it as U*Nguide*Np
+        ldw <- array(log_dmeas_weights, c(nunits,Nguide,Np))
+
+        ## average dmeas (natural scale) over Nguide sims, then take log,
+	## and then sum over 1:U (for each particle)
+        log_fcst_lik <- colSums(log(apply(exp(ldw),c(1,3),sum)/Nguide)) 
         log_resamp_weights <- log_fcst_lik*discount_factor
         log_guide_fun = log_guide_fun + log_resamp_weights
       }
+      rm(skel)
       log_s_not_1_weights <- log_guide_fun - log_filter_guide_fun
       if (!(s==1 & nt!=0)){
         log_weights <- log_s_not_1_weights
@@ -683,10 +668,7 @@ igirf.bootgirf <- function (object, params, Ninter, lookahead, Nguide,
             log=TRUE,
             .gnsi=gnsi
           ),
-          error = function (e) {
-            stop(ep,"error in calculation of log_meas_weights: ",
-              conditionMessage(e),call.=FALSE)
-          }
+          error = function (e) pStop_("error in igirf_bootstrap calculation of log_meas_weights: ", conditionMessage(e))
         )
         gnsi <- FALSE
         log_weights <- as.numeric(log_meas_weights) + log_s_not_1_weights
@@ -708,19 +690,18 @@ igirf.bootgirf <- function (object, params, Ninter, lookahead, Nguide,
           .Call(girf_computations,
             x=X,
             params=params,
-            Np=Np[nt+1],
+            Np=Np,
             trackancestry=TRUE,
             doparRS=TRUE,
             weights=weights,
             lgps=log_guide_fun,
-            fsv=array(0,dim=c(length(unit_names(object)), lookahead_steps, Np[1])), # bootgirf2 doesn't use fsv, set to an arbitrary val.
+	    ## bootgirf2 doesn't use fsv, set to an arbitrary value
+            fsv=array(0,dim=c(length(unit_names(object)), lookahead_steps, Np)), 
             tol=tol
           ),
-          error = function (e) {
-            stop(ep,conditionMessage(e),call.=FALSE) # nocov
-          }
+          error = function (e) pStop_("error in igirf_bootsrap computations : ", conditionMessage(e)) 
         )
-        guidesim_index = guidesim_index[xx$ancestry] # update guidesim index
+        guidesim_index = guidesim_index[xx$ancestry] 
         cond.loglik[nt+1, s] <- xx$loglik + max_log_weights
         x <- xx$states
         log_filter_guide_fun <- xx$logfilterguides
@@ -740,27 +721,28 @@ igirf.bootgirf <- function (object, params, Ninter, lookahead, Nguide,
     Nguide=Nguide,
     lookahead=lookahead,
     paramMatrix=params,
-    Np=Np[1],
+    Np=Np,
     tol=tol,
     loglik=sum(cond.loglik)
   )
 }
 
-illegal_dunit_measure_error <- function(time, lik, datvals, states, params){
-  showvals <- c(time=time,lik=lik,datvals,states,params)
-  m1 <- formatC(names(showvals),preserve.width="common")
-  m2 <- formatC(showvals,digits=6,width=12,format="g",preserve.width="common")
-  stop(
-    sQuote("dunit_measure")," returns illegal value.\n",
-    "Likelihood, data, states, and parameters are:\n",
-    paste0(m1,": ",m2,collapse="\n")
-  )
-}
+## uncomment for debugging: tracking down particles with NA weight
+## illegal_dunit_measure_error <- function(time, lik, datvals, states, params){
+##   showvals <- c(time=time,lik=lik,datvals,states,params)
+##   m1 <- formatC(names(showvals),preserve.width="common")
+##   m2 <- formatC(showvals,digits=6,width=12,format="g",preserve.width="common")
+##   stop(
+##     sQuote("dunit_measure")," returns illegal value.\n",
+##     "Likelihood, data, states, and parameters are:\n",
+##     paste0(m1,": ",m2,collapse="\n")
+##   )
+## }
 
 igirf_rw_sd_times <- function(times, Ninter){
   rw_sd_times <- c()
   for(i in seq(length(times)-1)) rw_sd_times <- c(
-                                   rw_sd_times, seq(from=times[i],to=times[i+1],length.out=Ninter+1))
+    rw_sd_times, seq(from=times[i],to=times[i+1],length.out=Ninter+1))
   return(rw_sd_times)
 }
 
