@@ -46,9 +46,9 @@ setMethod(
                object,
                x,
                time,
-               unit,
+               as.integer(unit-1),
                params,
-               TRUE)[unit,,1,drop=FALSE]
+               TRUE)
     pompUnload(object)
     out
   }
